import router from "../router";
import { userManager } from "../services/authentication";
import { addBearerTokenInterceptor, removeBearerTokenInterceptor } from "../services/http";
import { get, put } from "../services/persistence";

const identity = {
    state: () => ({
        user: null,
        loading: true,
        callbackPath: null,
    }),
    getters: {
        isAuthenticated(state) {
            return state.user ? !state.user.expired : false;
        },
        username(state) {
            return (state.user && state.user.profile.name) ? state.user.profile.name : null;
        },
        authCallbackLocation(state) {
            return state.callbackPath ? state.callbackPath : get("callbackPath");
        }
    },
    mutations: {
        login(state, { user }) {
            if (!user) return;
            state.user = user;
            addBearerTokenInterceptor(user.access_token);
        },
        logout(state) {
            state.user = null;
            removeBearerTokenInterceptor();
        },
        beginAuthenticating(state) {
            state.loading = true;
        },
        endAuthenticating(state) {
            state.loading = false;
        },
        authCallbackLocation(state) {
            state.callbackPath = window.location.pathname;
            put("callbackPath", state.callbackPath);
        }
    },
    actions: {
        async loadUser(context) {
            context.commit("beginAuthenticating");
            const user = await userManager.getUser();
            if (user != null) {
                context.commit("login", { user });
            }
            context.commit("endAuthenticating");
        },
        async attemptSilentAuthentication(context) {
            if (context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.dispatch("loadUser");
            if (!context.getters.isAuthenticated) {
                try {
                    const user = await userManager.signinSilent({
                        redirect_uri: window.location.origin + "/authentication/silent-login-callback"
                    });

                    context.commit("login", { user });
                } catch { }
                context.commit("endAuthenticating");
            }
        },
        async beginAuthentication(context) {
            if (context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.dispatch("loadUser");
            if (!context.getters.isAuthenticated) {
                context.commit("authCallbackLocation");
                userManager.signinRedirect();
            }
        },
        async beginDeauthentication(context) {
            if (!context.getters.isAuthenticated) return;
            context.commit("beginAuthenticating");
            context.commit("authCallbackLocation");
            await userManager.removeUser();
            userManager.signoutRedirect();
        },
        async completeAuthentication(context, { user }) {
            if (!user) return;
            context.commit("login", { user });
            router.push(context.getters.authCallbackLocation);
            context.commit("endAuthenticating");
        },
        async completeDeauthentication(context) {
            if (!context.getters.isAuthenticated) return;
            context.commit("logout");
            router.push(context.getters.authCallbackLocation);
            context.commit("endAuthenticating");
        }
    },
};

export { identity };
